@@ -130,7 +130,13 @@ class DataLabelerColumn(BaseColumnProfiler):
                                  "profiles.")
         
         self.assert_equal_conditions(self, other)
-        merged_profile = DataLabelerColumn(None)
+
+        # recreate options so the DataLabeler is transferred and not duplicated
+        options = DataLabelerOptions()
+        options.max_sample_size = self._max_sample_size
+        options.data_labeler_object = self.data_labeler
+
+        merged_profile = DataLabelerColumn(self.name, options)
         BaseColumnProfiler._add_helper(merged_profile, self, other)
 
         #Set all common variables
