@@ -11,13 +11,11 @@ from __future__ import division
 import copy
 import random
 import re
-import hashlib
 from collections import OrderedDict
 import warnings
 
-import multiprocessing as mp
-
 import pandas as pd
+import numpy as np
 
 from . import utils
 from .. import data_readers
@@ -25,12 +23,12 @@ from .column_profile_compilers import ColumnPrimitiveTypeProfileCompiler, \
     ColumnStatsProfileCompiler, ColumnDataLabelerCompiler
 from ..labelers.data_labelers import DataLabeler
 from .helpers.report_helpers import calculate_quantiles, _prepare_report
-from .profiler_options import ProfilerOptions, StructuredOptions, \
-    DataLabelerOptions
+from .profiler_options import ProfilerOptions, StructuredOptions
+
 
 class StructuredDataProfile(object):
 
-    def __init__(self, df_series, sample_size=None, min_sample_size=5000,
+    def __init__(self, df_series=None, sample_size=None, min_sample_size=5000,
                  sampling_ratio=0.2, min_true_samples=None,
                  sample_ids=None, pool=None, options=None):
         """
@@ -50,6 +48,7 @@ class StructuredDataProfile(object):
         :param options: Options for the structured profiler.
         :type options: StructuredOptions Object
         """
+        self.name = None
         self.options = options
         self._min_sample_size = min_sample_size
         self._sampling_ratio = sampling_ratio
@@ -57,49 +56,73 @@ class StructuredDataProfile(object):
         if self._min_true_samples is None:
             self._min_true_samples = 0
 
-        # if you create your own DF without giving the column name,
-        # it labels the name as an int64, however, if you try to
-        # `json.dump` an int64, it errors.
-        if isinstance(df_series.name, str) or df_series.name is None:
-            self.name = df_series.name
-        else:
-            self.name = int(df_series.name)
-
         self.sample_size = 0
         self.sample = list()
         self.null_count = 0
         self.null_types = list()
         self.null_types_index = {}
+        self.profiles = {}
+                         
+        if df_series is not None and len(df_series) > 0:
+            
+            if not sample_size:
+                sample_size = self._get_sample_size(df_series)
+            if sample_size < len(df_series):
+                warnings.warn("The data will be profiled with a sample size of {}. "
+                              "All statistics will be based on this subsample and "
+                              "not the whole dataset.".format(sample_size))
                 
-        if not sample_size:
-            sample_size = self._get_sample_size(df_series)
-        if sample_size < len(df_series):
-            warnings.warn("The data will be profiled with a sample size of {}. "
-                          "All statistics will be based on this subsample and "
-                          "not the whole dataset.".format(sample_size))
-        clean_sampled_df, base_stats = \
-            self.get_base_props_and_clean_null_params(
-                df_series, sample_size, sample_ids=sample_ids)
-        self._update_base_stats(base_stats)
+            clean_sampled_df, base_stats = \
+                self.clean_data_and_get_base_stats(
+                    df_series=df_series, sample_size=sample_size,
+                    min_true_samples=self._min_true_samples, sample_ids=sample_ids)
+            self.update_column_profilers(clean_sampled_df, pool)
+            self._update_base_stats(base_stats)
 
-        self.profiles = {
-            'data_type_profile':
-            ColumnPrimitiveTypeProfileCompiler(
-                clean_sampled_df, self.options, pool),
-            'data_stats_profile':
-            ColumnStatsProfileCompiler(
-                clean_sampled_df, self.options, pool)
-        }
+    def update_column_profilers(self, clean_sampled_df, pool):
+        """
+        Calculates type statistics and labels dataset
+        
+        :param clean_sampled_df: sampled series with none types dropped
+        :type clean_sampled_df: Pandas.Series
+        :param pool: pool utilized for multiprocessing
+        :type pool: multiprocessing.pool
+        """
 
-        use_data_labeler = True
-        if options and isinstance(options, StructuredOptions):
-            use_data_labeler = options.data_labeler.is_enabled
+        if self.name is None:
+            self.name = clean_sampled_df.name
+        if self.name != clean_sampled_df.name:
+            raise ValueError(
+                'Column names have changed, col {} does not match prior name {}',
+                clean_sampled_df.name, self.name
+            )
+        
+        # First run, create the compilers
+        if self.profiles is None or len(self.profiles) == 0:
+            self.profiles = {
+                'data_type_profile':
+                ColumnPrimitiveTypeProfileCompiler(
+                    clean_sampled_df, self.options, pool),
+                'data_stats_profile':
+                ColumnStatsProfileCompiler(
+                    clean_sampled_df, self.options, pool)
+            }
+        
+            use_data_labeler = True
+            if self.options and isinstance(self.options, StructuredOptions):
+                use_data_labeler = self.options.data_labeler.is_enabled
 
-        if use_data_labeler:
-            self.profiles.update({
-                'data_label_profile':
-                ColumnDataLabelerCompiler(clean_sampled_df, self.options)
-            })
+            if use_data_labeler:
+                self.profiles.update({
+                    'data_label_profile':
+                    ColumnDataLabelerCompiler(
+                        clean_sampled_df, self.options, pool)
+                })
+        else:
+
+            # Profile compilers being updated
+            for profile in self.profiles.values():
+                profile.update_profile(clean_sampled_df, pool)
 
     def __add__(self, other):
         """
@@ -124,16 +147,20 @@ class StructuredDataProfile(object):
             min_sample_size=max(self._min_sample_size, other._min_sample_size),
             sampling_ratio=max(self._sampling_ratio, other._sampling_ratio),
             min_true_samples=max(self._min_true_samples,
-                                 other._min_true_samples))
+                                 other._min_true_samples),
+            options=self.options,
+        )
 
         merged_profile.name = self.name
         merged_profile._update_base_stats(
-            {"sample": self.sample, 'sample_size': self.sample_size,
+            {"sample": self.sample,
+             "sample_size": self.sample_size,
              "null_count": self.null_count,
              "null_types": copy.deepcopy(self.null_types_index)}
         )
         merged_profile._update_base_stats(
-            {"sample": other.sample, 'sample_size': other.sample_size,
+            {"sample": other.sample,
+             "sample_size": other.sample_size,
              "null_count": other.null_count,
              "null_types": copy.deepcopy(other.null_types_index)}
         )
@@ -150,8 +177,13 @@ class StructuredDataProfile(object):
         unordered_profile = dict()
         for profile in self.profiles.values():
             utils.dict_merge(unordered_profile, profile.profile)
+
+        name = self.name
+        if isinstance(self.name, np.integer):
+            name = int(name)
+            
         unordered_profile.update({
-            "column_name": self.name,
+            "column_name": name,
             "samples": self.sample,
         })
                 
@@ -162,7 +194,7 @@ class StructuredDataProfile(object):
                 "null_types": self.null_types,
                 "null_types_index": self.null_types_index,
                 "data_type_representation":
-                    unordered_profile["data_type_representation"]
+                unordered_profile["data_type_representation"]
             })
 
         dict_order = [
@@ -198,7 +230,6 @@ class StructuredDataProfile(object):
                 null_rows.sort()
             self.null_types_index.setdefault(null_type, set()).update(null_rows)
 
-
     def update_profile(self, df_series, sample_size=None,
                        min_true_samples=None, sample_ids=None,
                        pool=None):
@@ -221,18 +252,15 @@ class StructuredDataProfile(object):
             sample_size = len(df_series)
         if not sample_size:
             sample_size = self._get_sample_size(df_series)
-        clean_sampled_df, base_stats = \
-            self.get_base_props_and_clean_null_params(
-                df_series, sample_size,
-                min_true_samples=min_true_samples,
-                sample_ids=sample_ids
-            )
+        if not min_true_samples:
+            min_true_samples = self._min_true_samples
+        
+        clean_sampled_df, base_stats = self.clean_data_and_get_base_stats(
+            df_series=df_series, sample_size=sample_size,
+            min_true_samples=min_true_samples, sample_ids=sample_ids)
+
         self._update_base_stats(base_stats)
-
-        # Profile compilers being updated
-        for profile in self.profiles.values():
-            profile.update_profile(clean_sampled_df, pool)
-
+        self.update_column_profilers(clean_sampled_df, pool)
 
     def _get_sample_size(self, df_series):
         """
@@ -250,9 +278,10 @@ class StructuredDataProfile(object):
 
     # TODO: flag column name with null values and potentially return row
     #  index number in the error as well
-    def get_base_props_and_clean_null_params(self, df_series, sample_size,
-                                             min_true_samples=None,
-                                             sample_ids=None):
+    @staticmethod
+    def clean_data_and_get_base_stats(df_series, sample_size,
+                                      min_true_samples=None,
+                                      sample_ids=None):
         """
         Identify null characters and return them in a dictionary as well as
         remove any nulls in column.
@@ -281,19 +310,20 @@ class StructuredDataProfile(object):
             "__*": NO_FLAG,
         }
         
+        if min_true_samples is None:
+            min_true_samples = 0
+        
         len_df = len(df_series)
         if not len_df:
             return df_series, {
-                "sample_size": 0, "null_count": 0, "null_types": dict(),
-                "sample": []}
-
-        if min_true_samples is None:
-            min_true_samples = self._min_true_samples
+                "sample_size": 0, "null_count": 0,
+                "null_types": dict(), "sample": []
+            }
 
         # Pandas reads empty values in the csv files as nan
         df_series = df_series.apply(str)
 
-        # Select generator depending if sample_ids availablity
+        # Select generator depending if sample_ids availability
         if sample_ids is None:
             sample_ind_generator = utils.shuffle_in_chunks(
                 len_df, chunk_size=sample_size)
@@ -302,44 +332,45 @@ class StructuredDataProfile(object):
                 sample_ids[0], chunk_size=sample_size)
             
         na_columns = dict()
-        true_sample_list = list()
+        true_sample_list = set()
         total_sample_size = 0
+        query = '|'.join(null_values_and_flags.keys())
+        regex = f"^(?:{(query)})$"
         for chunked_sample_ids in sample_ind_generator:
             total_sample_size += len(chunked_sample_ids)
-
-            df_series_subset = df_series.iloc[chunked_sample_ids]
-
-            query = '(' + '|'.join(null_values_and_flags.keys()) + ')'
-            reg_ex_na = f"^{(query)}$"
-            matching_na_elements = df_series_subset.str.contains(
-                reg_ex_na, flags=re.IGNORECASE)
-
-            for row, elem in matching_na_elements.items():
-                if elem:
-                    # Since df_series_subset[row] is mutable,
-                    # need to make new var
-                    row_value = str(df_series_subset[row])
-                    na_columns.setdefault(row_value, list()).append(row)
-                    
-            # Drop the values that matched regex_na
-            df_series_subset = df_series_subset[~matching_na_elements]
             
-            true_sample_list += df_series_subset.index.tolist()
+            # Find subset of series based on randomly selected ids
+            df_subset = df_series.iloc[chunked_sample_ids]
 
-            if len(true_sample_list) >= min_true_samples and total_sample_size:
+            # Query should search entire cell for all elements at once
+            matches = df_subset.str.match(regex, flags=re.IGNORECASE)
+            
+            # Split series into None samples and true samples
+            true_sample_list.update(df_subset[~matches].index)
+
+            # Iterate over all the Nones
+            for index, cell in df_subset[matches].items():
+                na_columns.setdefault(cell, list()).append(index)
+            
+            # Ensure minimum number of true samples met
+            # and if total_sample_size >= sample size, exit
+            if len(true_sample_list) >= min_true_samples \
+                    and total_sample_size >= sample_size:
                 break
             
         # close the generator in case it is not exhausted.
         if sample_ids is None:
             sample_ind_generator.close()
 
-        df_series = df_series.loc[sorted(true_sample_list)]
-        non_na = len(df_series)
-        total_na = total_sample_size - non_na
+        # If min_true_samples exists, sort
+        if min_true_samples > 0:
+            true_sample_list = sorted(true_sample_list)
+
+        # Split out true values for later utilization
+        df_series = df_series.loc[true_sample_list]
+        total_na = total_sample_size - len(true_sample_list)
 
         base_stats = {
-            # TODO: Is this correct? used to be actual sample size, including
-            #  NANs, what now?
             "sample_size": total_sample_size,
             "null_count": total_na,
             "null_types": na_columns,
@@ -352,7 +383,7 @@ class StructuredDataProfile(object):
 
 class Profiler(object):
 
-    def __init__(self, data, samples_per_update=None, min_true_samples=None, 
+    def __init__(self, data, samples_per_update=None, min_true_samples=0, 
                  profiler_options=None):
         """
         Instantiate the Profiler class
@@ -388,6 +419,11 @@ class Profiler(object):
         self._min_true_samples = min_true_samples
         self._profile = dict()
 
+        # matches structured data profile
+        # TODO: allow set via options
+        self._sampling_ratio = 0.2
+        self._min_sample_size = 5000
+
         if isinstance(data, data_readers.text_data.TextData):
             raise TypeError("Cannot provide TextData object to Profiler")
 
@@ -395,20 +431,22 @@ class Profiler(object):
         data_labeler_options = self.options.structured_options.data_labeler
         if data_labeler_options.is_enabled \
                 and data_labeler_options.data_labeler_object is None:
-            
+
             try:
-                
+
                 data_labeler = DataLabeler(
                     labeler_type='structured',
                     dirpath=data_labeler_options.data_labeler_dirpath,
                     load_options=None)
-                self.options.set({'data_labeler.data_labeler_object': data_labeler})
-                
+                self.options.set(
+                    {'data_labeler.data_labeler_object': data_labeler})
+
             except Exception as e:
                 utils.warn_on_profile('data_labeler', e)
                 self.options.set({'data_labeler.is_enabled': False})
 
-        self.update_profile(data)
+        if len(data):
+            self.update_profile(data)
 
     def __add__(self, other):
         """
@@ -431,9 +469,9 @@ class Profiler(object):
                              'profiles and cannot be added together.')
         merged_profile = Profiler(
             data=pd.DataFrame([]), samples_per_update=self._samples_per_update,
-            min_true_samples=self._min_true_samples, profiler_options=None
+            min_true_samples=self._min_true_samples,
+            profiler_options=self.options
         )
-        merged_profile.options = self.options
         merged_profile.encoding = self.encoding \
             if self.encoding == other.encoding else 'multiple files'
         merged_profile.file_type = self.file_type \
@@ -456,6 +494,23 @@ class Profiler(object):
     def profile(self):
         return self._profile
 
+    def _get_sample_size(self, data):
+        """
+        Determines the minimum sampling size for detecting column type.
+
+        :param data: data to be profiled
+        :type data: Union[data_readers.base_data.BaseData, pandas.DataFrame]
+        :return: integer sampling size
+        :rtype: int
+        """
+        if self._samples_per_update:
+            return self._samples_per_update
+
+        len_data = len(data)
+        if len_data <= self._min_sample_size:
+            return int(len_data)
+        return max(int(self._sampling_ratio * len_data), self._min_sample_size)
+
     @property
     def _max_col_samples_used(self):
         """
@@ -467,6 +522,16 @@ class Profiler(object):
             samples_used = max(samples_used, col.sample_size)
         return samples_used
 
+    @property
+    def _min_col_samples_used(self):
+        """
+        Calculates and returns the number of rows that were completely sampled
+        i.e. every column in the Profile was read up to this row (possibly
+        further in some cols)
+        """
+        return min([self._profile[col].sample_size
+                    for col in self._profile], default=0)
+
     def report(self, report_options=None):
         if not report_options:
             report_options = {
@@ -507,17 +572,17 @@ class Profiler(object):
         return len(self.hashed_row_dict) / self.total_samples
 
     def _get_row_is_null_ratio(self):
-        return 0 if self._max_col_samples_used == 0 \
-            else self.row_is_null_count / self._max_col_samples_used
+        return 0 if self._min_col_samples_used in {0, None} \
+            else self.row_is_null_count / self._min_col_samples_used
 
     def _get_row_has_null_ratio(self):
-        return 0 if self._max_col_samples_used == 0 \
-            else self.row_has_null_count / self._max_col_samples_used
+        return 0 if self._min_col_samples_used in {0, None} \
+            else self.row_has_null_count / self._min_col_samples_used
 
     def _get_duplicate_row_count(self):
         return self.total_samples - len(self.hashed_row_dict)
 
-    def _update_row_statistics(self, data):
+    def _update_row_statistics(self, data, sample_ids=None):
         """
         Iterate over the provided dataset row by row and calculate
         the row statistics. Specifically, number of unique rows,
@@ -526,7 +591,13 @@ class Profiler(object):
 
         :param data: a dataset
         :type data: pandas.DataFrame
+        :param sample_ids: list of indices in order they were sampled in data
+        :type sample_ids: list(int)
         """
+
+        if not isinstance(data, pd.DataFrame):
+            raise ValueError("Cannot calculate row statistics on data that is"
+                             "not a DataFrame")
         
         self.total_samples += len(data)
         self.hashed_row_dict = dict.fromkeys(
@@ -543,6 +614,12 @@ class Profiler(object):
             if null_type_dict:
                 null_row_indices = set.union(*null_type_dict.values())
 
+            if sample_ids is not None:
+                # If sample ids provided, only consider nulls in rows that
+                # were fully sampled
+                null_row_indices = null_row_indices.intersection(
+                    data.index[sample_ids[:self._min_col_samples_used]])
+
             # Find the common null indices between the columns
             if first_col_flag:
                 null_rows = null_row_indices
@@ -554,7 +631,7 @@ class Profiler(object):
 
         self.row_has_null_count += len(null_in_row_count)
         self.row_is_null_count += len(null_rows)
-        
+
     def update_profile(self, data, sample_size=None, min_true_samples=None):
         """
         Update the profile for data provided. User can specify the sample
@@ -569,20 +646,11 @@ class Profiler(object):
         :type min_true_samples
         :return: None
         """
-        if not sample_size:
-            sample_size = self._samples_per_update
-        if not min_true_samples:
-            min_true_samples = self._min_true_samples
         if isinstance(data, data_readers.base_data.BaseData):
-            self._profile = self._update_profile_from_chunk(
-                data.data, self._profile, sample_size, min_true_samples, self.options)
-            self._update_row_statistics(data.data)
             self.encoding = data.file_encoding
             self.file_type = data.data_type
+            data = data.data
         elif isinstance(data, pd.DataFrame):
-            self._profile = self._update_profile_from_chunk(
-                data, self._profile, sample_size, min_true_samples, self.options)
-            self._update_row_statistics(data)
             self.file_type = str(data.__class__)
         else:
             raise ValueError(
@@ -590,16 +658,21 @@ class Profiler(object):
                 "pd.DataFrame."
             )
 
-    @staticmethod
-    def _update_profile_from_chunk(df, profile=None, sample_size=None,
+        if not min_true_samples:
+            min_true_samples = self._min_true_samples
+        if not sample_size:
+            sample_size = self._get_sample_size(data)
+
+        self._update_profile_from_chunk(
+            data, sample_size, min_true_samples, self.options)
+
+    def _update_profile_from_chunk(self, df, sample_size=None,
                                    min_true_samples=None, options=None):
         """
         Iterate over the columns of a dataset and identify its parameters.
         
         :param df: a dataset
         :type df: pandas.DataFrame
-        :param profile: list of profiled columns [BaseColumnProfiler subclasses]
-        :type profile: list
         :param sample_size: number of samples for df to use for profiling
         :type sample_size: int
         :param min_true_samples: minimum number of true samples required
@@ -609,8 +682,6 @@ class Profiler(object):
         :return: list of column profile base subclasses
         :rtype: list(BaseColumnProfiler)
         """
-        if not profile:
-            profile = OrderedDict()
 
         if len(df.columns) != len(df.columns.unique()):
             raise ValueError('`Profiler` does not currently support data which '
@@ -622,52 +693,133 @@ class Profiler(object):
             def tqdm(l):
                 for i, e in enumerate(l):
                     print("Processing Column {}/{}".format(i+1, len(l)))
-                    yield e                    
+                    yield e
 
-        # Shuffle indices ones and share with columns
+        # Shuffle indices once and share with columns
         sample_ids = [*utils.shuffle_in_chunks(len(df), len(df))]
-
-        pool = None
-        if options.structured_options.multiprocess.is_enabled:
-            cpu_count = 1
-            try:
-                cpu_count = mp.cpu_count()
-            except NotImplementedError as e:
-                cpu_count = 1
-
-            # No additional advantage beyond 8 processes
-            # Always leave 1 cores free
-            if cpu_count > 2:
-                cpu_count = min(cpu_count-1, 8)
-                pool = mp.Pool(cpu_count)
-                print("Utilizing",cpu_count, "processes for profiling")
         
-        for col in tqdm(df.columns):
-            if col in profile:
-                column_profile = profile[col]
-                column_profile.update_profile(
-                    df[col],
-                    sample_size=sample_size,
-                    min_true_samples=min_true_samples,
-                    sample_ids=sample_ids,
-                    pool=pool
-                )
-            else:
+        # If there are no minimum true samples, you can sort to save time
+        if min_true_samples in [None, 0]:
+            # If there's a sample size, truncate
+            if sample_size is not None:
+                sample_ids[0] = sample_ids[0][:sample_size]
+            # Sort the sample_ids and replace prior
+            sample_ids[0] = sorted(sample_ids[0])
+
+        # Numpy arrays allocate to heap and can be shared between processes
+        # Non-locking multiprocessing fails on machines without POSIX (windows)
+        # The function handles that situation, but will be single process
+        # Newly introduced features (python3.8) improves the situation
+        sample_ids = np.array(sample_ids)
+
+        # Create structured profile objects
+        new_cols = set()
+        for col in df.columns:
+            if col not in self._profile:
                 structured_options = None
                 if options and options.structured_options:
                     structured_options = options.structured_options
-                profile[col] = StructuredDataProfile(
-                    df[col],
+                self._profile[col] = StructuredDataProfile(
                     sample_size=sample_size,
                     min_true_samples=min_true_samples,
                     sample_ids=sample_ids,
-                    pool=pool,
                     options=structured_options
                 )
+                new_cols.add(col)
+                
+        # Generate pool and estimate datasize
+        pool = None
+        if options.structured_options.multiprocess.is_enabled:
+            est_data_size = df[:50000].memory_usage(index=False, deep=True).sum()
+            est_data_size = (est_data_size / min(50000, len(df))) * len(df)
+            pool, pool_size = utils.generate_pool(
+                max_pool_size=None, data_size=est_data_size, cols=len(df.columns))
+
+        # Format the data
+        notification_str = "Finding the Null values in the columns..."        
+        if pool and len(new_cols) > 0:
+            notification_str += " (with " + str(pool_size) + " processes)"
+        
+        clean_sampled_dict = {}
+        multi_process_dict = {}
+        single_process_list = set()
+        if not sample_size: sample_size = len(df)
+        if sample_size < len(df):
+            warnings.warn("The data will be profiled with a sample size of {}. "
+                          "All statistics will be based on this subsample and "
+                          "not the whole dataset.".format(sample_size))
 
         if pool is not None:
-            pool.close() # Close pool for new tasks
-            pool.join() # Wait for all workers to complete
-            
-        return profile
+            # Create a bunch of simultaneous column conversions
+            for col in df.columns:
+                if min_true_samples is None:
+                    min_true_samples = self._profile[col]._min_true_samples
+                try:
+                    multi_process_dict[col] = pool.apply_async(
+                        self._profile[col].clean_data_and_get_base_stats,
+                        (df[col], sample_size, min_true_samples, sample_ids))
+                except Exception as e:
+                    print(e)
+                    single_process_list.add(col)
+                
+            # Iterate through multiprocessed columns collecting results
+            print(notification_str)
+            for col in tqdm(multi_process_dict.keys()):
+                try:
+                    clean_sampled_dict[col], base_stats = \
+                        multi_process_dict[col].get()
+                    self._profile[col]._update_base_stats(base_stats)
+                except Exception as e:
+                    print(e)
+                    single_process_list.add(col)
 
+            # Clean up any columns which errored
+            if len(single_process_list) > 0:
+                print("Errors in multiprocessing occured:",
+                      len(single_process_list), "errors, reprocessing...")
+                for col in tqdm(single_process_list):
+                    if min_true_samples is None:
+                        min_true_samples = self._profile[col]._min_true_samples
+                    clean_sampled_dict[col], base_stats = \
+                        self._profile[col].clean_data_and_get_base_stats(
+                            df[col], sample_size, min_true_samples, sample_ids)
+                    self._profile[col]._update_base_stats(base_stats)
+            
+            pool.close()  # Close pool for new tasks
+            pool.join()  # Wait for all workers to complete
+
+        else:  # No pool
+            print(notification_str)
+            for col in tqdm(df.columns):
+                if min_true_samples is None:
+                    min_true_samples = self._profile[col]._min_true_samples
+                clean_sampled_dict[col], base_stats = \
+                    self._profile[col].clean_data_and_get_base_stats(
+                        df_series=df[col], sample_size=sample_size,
+                        min_true_samples=min_true_samples, sample_ids=sample_ids
+                    )
+                self._profile[col]._update_base_stats(base_stats)
+            
+        # Process and label the data
+        notification_str = "Calculating the statistics... "
+        pool = None
+        if options.structured_options.multiprocess.is_enabled:
+            pool, pool_size = utils.generate_pool(4, est_data_size)
+            if pool:
+                notification_str += " (with " + str(pool_size) + " processes)"
+        print(notification_str)
+        
+        for col in tqdm(df.columns):
+            self._profile[col].update_column_profilers(
+                clean_sampled_dict[col], pool)
+
+        if pool is not None:
+            pool.close()  # Close pool for new tasks
+            pool.join()  # Wait for all workers to complete
+
+        # Only pass along sample ids if necessary
+        samples_for_row_stats = None
+        if min_true_samples not in [None, 0]:
+            samples_for_row_stats = sample_ids[0]
+
+        self._update_row_statistics(df, samples_for_row_stats)
