@@ -7,6 +7,7 @@ import pandas as pd
 import numpy as np
 
 from dataprofiler.profilers import NumericStatsMixin
+from dataprofiler.profilers.profiler_options import NumericalOptions
 
 
 test_root_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
@@ -27,6 +28,25 @@ class TestColumn(NumericStatsMixin):
 
 class TestNumericStatsMixin(unittest.TestCase):
 
+    @mock.patch.multiple(NumericStatsMixin, __abstractmethods__=set(),
+                         _filter_properties_w_options=mock.MagicMock(
+                             return_value=None),
+                         create=True)
+    def test_base(self):
+
+        # validate requires NumericalOptions
+        with self.assertRaisesRegex(ValueError,
+                                    "NumericalStatsMixin parameter 'options' "
+                                    "must be of type NumericalOptions."):
+            profile = NumericStatsMixin(options='bad options')
+
+        try:
+            # validate doesn't fail
+            profile = NumericStatsMixin()
+            profile = NumericStatsMixin(NumericalOptions())
+        except Exception as e:
+            self.fail(e)
+
     def test_check_float(self):
         """
         Checks if number is float.
@@ -180,8 +200,8 @@ class TestNumericStatsMixin(unittest.TestCase):
         other2.histogram_selection = "auto"
         other1.histogram_bin_method_names = ['auto']
         other2.histogram_bin_method_names = ['auto']
-        other1.histogram_methods['auto']['histogram'] = mock_histogram
-        other2.histogram_methods['auto']['histogram'] = mock_histogram
+        other1._stored_histogram['histogram'] = mock_histogram
+        other2._stored_histogram['histogram'] = mock_histogram
         other1.histogram_selection = 'auto'
 
         time_array = [float(i) for i in range(2, 0, -1)]
@@ -246,7 +266,7 @@ class TestNumericStatsMixin(unittest.TestCase):
             self.assertEqual(expected, num_profiler.times)
 
             # Validate _get_histogram_and_quantiles is timed.
-            expected['histogram_and_quantiles'] = 15.0
+            expected['histogram_and_quantiles'] = 1.0
             num_profiler._get_histogram_and_quantiles(
                 df_series, prev_dependent_properties, subset_properties)
             self.assertEqual(expected, num_profiler.times)
