@@ -25,7 +25,8 @@ class TestDataProfiler(unittest.TestCase):
 
         test_dir = os.path.join(MODULE_PATH, 'data')
         cls.input_file_names = [
-            dict(path=os.path.join(test_dir, 'csv/aws_honeypot_marx_geo.csv'), type='csv'),
+            dict(path=os.path.join(test_dir, 'csv/aws_honeypot_marx_geo.csv'),
+                 type='csv'),
         ]
 
     def test_data_import(self):
@@ -81,8 +82,6 @@ class TestDataProfiler(unittest.TestCase):
 
     def test_no_tensorflow(self):
         import sys
-        import importlib
-        import types
         import pandas
         orig_import = __import__
         # necessary for any wrapper around the library to test if snappy caught
@@ -92,16 +91,20 @@ class TestDataProfiler(unittest.TestCase):
             if name == 'tensorflow':
                 raise ImportError('test')
             return orig_import(name, *args)
-                        
+
         with mock.patch('builtins.__import__', side_effect=import_mock):
 
-            with self.assertWarns(RuntimeWarning) as w:
-                import dataprofiler
-                df = pandas.DataFrame([[1, 2.0],[1, 2.2],[-1, 3]])
-                profile = dataprofiler.Profiler(df)
-        
-        warning_msg = "Partial Profiler Failure"
-        self.assertIn(warning_msg, str(w.warning))
+            with self.assertWarnsRegex(RuntimeWarning,
+                                       "Partial Profiler Failure"):
+                modules_with_tf = [
+                    'dataprofiler.labelers.character_level_cnn_model',
+                ]
+                for module in modules_with_tf:
+                    if module in sys.modules:
+                        del sys.modules[module]
+                df = pandas.DataFrame([[1, 2.0], [1, 2.2], [-1, 3]])
+                profile = Profiler(df)
+
 
 if __name__ == '__main__':
     unittest.main()
